# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["WARPConnectorDeleteResponse", "Connection"]


class Connection(BaseModel):
    id: Optional[str] = None
    """UUID of the Cloudflare Tunnel connection."""

    client_id: Optional[str] = None
    """UUID of the Cloudflare Tunnel connector."""

    client_version: Optional[str] = None
    """The cloudflared version used to establish this connection."""

    colo_name: Optional[str] = None
    """The Cloudflare data center used for this connection."""

    is_pending_reconnect: Optional[bool] = None
    """
    Cloudflare continues to track connections for several minutes after they
    disconnect. This is an optimization to improve latency and reliability of
    reconnecting. If `true`, the connection has disconnected but is still being
    tracked. If `false`, the connection is actively serving traffic.
    """

    opened_at: Optional[datetime] = None
    """Timestamp of when the connection was established."""

    origin_ip: Optional[str] = None
    """The public IP address of the host running cloudflared."""

    uuid: Optional[str] = None
    """UUID of the Cloudflare Tunnel connection."""


class WARPConnectorDeleteResponse(BaseModel):
    """A Warp Connector Tunnel that connects your origin to Cloudflare's edge."""

    id: Optional[str] = None
    """UUID of the tunnel."""

    account_tag: Optional[str] = None
    """Cloudflare account ID"""

    connections: Optional[List[Connection]] = None
    """The Cloudflare Tunnel connections between your origin and Cloudflare's edge."""

    conns_active_at: Optional[datetime] = None
    """
    Timestamp of when the tunnel established at least one connection to Cloudflare's
    edge. If `null`, the tunnel is inactive.
    """

    conns_inactive_at: Optional[datetime] = None
    """
    Timestamp of when the tunnel became inactive (no connections to Cloudflare's
    edge). If `null`, the tunnel is active.
    """

    created_at: Optional[datetime] = None
    """Timestamp of when the resource was created."""

    deleted_at: Optional[datetime] = None
    """Timestamp of when the resource was deleted.

    If `null`, the resource has not been deleted.
    """

    metadata: Optional[object] = None
    """Metadata associated with the tunnel."""

    name: Optional[str] = None
    """A user-friendly name for a tunnel."""

    status: Optional[Literal["inactive", "degraded", "healthy", "down"]] = None
    """The status of the tunnel.

    Valid values are `inactive` (tunnel has never been run), `degraded` (tunnel is
    active and able to serve traffic but in an unhealthy state), `healthy` (tunnel
    is active and able to serve traffic), or `down` (tunnel can not serve traffic as
    it has no connections to the Cloudflare Edge).
    """

    tun_type: Optional[Literal["cfd_tunnel", "warp_connector", "warp", "magic", "ip_sec", "gre", "cni"]] = None
    """The type of tunnel."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncV4PagePagination, AsyncV4PagePagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.load_balancers import search_list_params
from ...types.load_balancers.search_list_response import SearchListResponse

__all__ = ["SearchesResource", "AsyncSearchesResource"]


class SearchesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SearchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SearchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SearchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SearchesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        query: str | Omit = omit,
        references: Literal["", "*", "referral", "referrer"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePagination[SearchListResponse]:
        """
        Search for Load Balancing resources.

        Args:
          account_id: Identifier.

          query: Search query term.

          references: The type of references to include. "\\**" to include both referral and referrer
              references. "" to not include any reference information.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/load_balancers/search", account_id=account_id),
            page=SyncV4PagePagination[SearchListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "query": query,
                        "references": references,
                    },
                    search_list_params.SearchListParams,
                ),
            ),
            model=SearchListResponse,
        )


class AsyncSearchesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSearchesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSearchesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSearchesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSearchesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        query: str | Omit = omit,
        references: Literal["", "*", "referral", "referrer"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SearchListResponse, AsyncV4PagePagination[SearchListResponse]]:
        """
        Search for Load Balancing resources.

        Args:
          account_id: Identifier.

          query: Search query term.

          references: The type of references to include. "\\**" to include both referral and referrer
              references. "" to not include any reference information.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/load_balancers/search", account_id=account_id),
            page=AsyncV4PagePagination[SearchListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "query": query,
                        "references": references,
                    },
                    search_list_params.SearchListParams,
                ),
            ),
            model=SearchListResponse,
        )


class SearchesResourceWithRawResponse:
    def __init__(self, searches: SearchesResource) -> None:
        self._searches = searches

        self.list = to_raw_response_wrapper(
            searches.list,
        )


class AsyncSearchesResourceWithRawResponse:
    def __init__(self, searches: AsyncSearchesResource) -> None:
        self._searches = searches

        self.list = async_to_raw_response_wrapper(
            searches.list,
        )


class SearchesResourceWithStreamingResponse:
    def __init__(self, searches: SearchesResource) -> None:
        self._searches = searches

        self.list = to_streamed_response_wrapper(
            searches.list,
        )


class AsyncSearchesResourceWithStreamingResponse:
    def __init__(self, searches: AsyncSearchesResource) -> None:
        self._searches = searches

        self.list = async_to_streamed_response_wrapper(
            searches.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .session import (
    SessionResource,
    AsyncSessionResource,
    SessionResourceWithRawResponse,
    AsyncSessionResourceWithRawResponse,
    SessionResourceWithStreamingResponse,
    AsyncSessionResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .browser.browser import (
    BrowserResource,
    AsyncBrowserResource,
    BrowserResourceWithRawResponse,
    AsyncBrowserResourceWithRawResponse,
    BrowserResourceWithStreamingResponse,
    AsyncBrowserResourceWithStreamingResponse,
)

__all__ = ["DevtoolsResource", "AsyncDevtoolsResource"]


class DevtoolsResource(SyncAPIResource):
    @cached_property
    def session(self) -> SessionResource:
        return SessionResource(self._client)

    @cached_property
    def browser(self) -> BrowserResource:
        return BrowserResource(self._client)

    @cached_property
    def with_raw_response(self) -> DevtoolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DevtoolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DevtoolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DevtoolsResourceWithStreamingResponse(self)


class AsyncDevtoolsResource(AsyncAPIResource):
    @cached_property
    def session(self) -> AsyncSessionResource:
        return AsyncSessionResource(self._client)

    @cached_property
    def browser(self) -> AsyncBrowserResource:
        return AsyncBrowserResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDevtoolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDevtoolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDevtoolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDevtoolsResourceWithStreamingResponse(self)


class DevtoolsResourceWithRawResponse:
    def __init__(self, devtools: DevtoolsResource) -> None:
        self._devtools = devtools

    @cached_property
    def session(self) -> SessionResourceWithRawResponse:
        return SessionResourceWithRawResponse(self._devtools.session)

    @cached_property
    def browser(self) -> BrowserResourceWithRawResponse:
        return BrowserResourceWithRawResponse(self._devtools.browser)


class AsyncDevtoolsResourceWithRawResponse:
    def __init__(self, devtools: AsyncDevtoolsResource) -> None:
        self._devtools = devtools

    @cached_property
    def session(self) -> AsyncSessionResourceWithRawResponse:
        return AsyncSessionResourceWithRawResponse(self._devtools.session)

    @cached_property
    def browser(self) -> AsyncBrowserResourceWithRawResponse:
        return AsyncBrowserResourceWithRawResponse(self._devtools.browser)


class DevtoolsResourceWithStreamingResponse:
    def __init__(self, devtools: DevtoolsResource) -> None:
        self._devtools = devtools

    @cached_property
    def session(self) -> SessionResourceWithStreamingResponse:
        return SessionResourceWithStreamingResponse(self._devtools.session)

    @cached_property
    def browser(self) -> BrowserResourceWithStreamingResponse:
        return BrowserResourceWithStreamingResponse(self._devtools.browser)


class AsyncDevtoolsResourceWithStreamingResponse:
    def __init__(self, devtools: AsyncDevtoolsResource) -> None:
        self._devtools = devtools

    @cached_property
    def session(self) -> AsyncSessionResourceWithStreamingResponse:
        return AsyncSessionResourceWithStreamingResponse(self._devtools.session)

    @cached_property
    def browser(self) -> AsyncBrowserResourceWithStreamingResponse:
        return AsyncBrowserResourceWithStreamingResponse(self._devtools.browser)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .dlp import (
    DLPResource,
    AsyncDLPResource,
    DLPResourceWithRawResponse,
    AsyncDLPResourceWithRawResponse,
    DLPResourceWithStreamingResponse,
    AsyncDLPResourceWithStreamingResponse,
)
from .email import (
    EmailResource,
    AsyncEmailResource,
    EmailResourceWithRawResponse,
    AsyncEmailResourceWithRawResponse,
    EmailResourceWithStreamingResponse,
    AsyncEmailResourceWithStreamingResponse,
)
from .limits import (
    LimitsResource,
    AsyncLimitsResource,
    LimitsResourceWithRawResponse,
    AsyncLimitsResourceWithRawResponse,
    LimitsResourceWithStreamingResponse,
    AsyncLimitsResourceWithStreamingResponse,
)
from .entries import (
    EntriesResource,
    AsyncEntriesResource,
    EntriesResourceWithRawResponse,
    AsyncEntriesResourceWithRawResponse,
    EntriesResourceWithStreamingResponse,
    AsyncEntriesResourceWithStreamingResponse,
)
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .patterns import (
    PatternsResource,
    AsyncPatternsResource,
    PatternsResourceWithRawResponse,
    AsyncPatternsResourceWithRawResponse,
    PatternsResourceWithStreamingResponse,
    AsyncPatternsResourceWithStreamingResponse,
)
from .profiles import (
    ProfilesResource,
    AsyncProfilesResource,
    ProfilesResourceWithRawResponse,
    AsyncProfilesResourceWithRawResponse,
    ProfilesResourceWithStreamingResponse,
    AsyncProfilesResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .payload_logs import (
    PayloadLogsResource,
    AsyncPayloadLogsResource,
    PayloadLogsResourceWithRawResponse,
    AsyncPayloadLogsResourceWithRawResponse,
    PayloadLogsResourceWithStreamingResponse,
    AsyncPayloadLogsResourceWithStreamingResponse,
)

__all__ = [
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
    "PatternsResource",
    "AsyncPatternsResource",
    "PatternsResourceWithRawResponse",
    "AsyncPatternsResourceWithRawResponse",
    "PatternsResourceWithStreamingResponse",
    "AsyncPatternsResourceWithStreamingResponse",
    "PayloadLogsResource",
    "AsyncPayloadLogsResource",
    "PayloadLogsResourceWithRawResponse",
    "AsyncPayloadLogsResourceWithRawResponse",
    "PayloadLogsResourceWithStreamingResponse",
    "AsyncPayloadLogsResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "EmailResource",
    "AsyncEmailResource",
    "EmailResourceWithRawResponse",
    "AsyncEmailResourceWithRawResponse",
    "EmailResourceWithStreamingResponse",
    "AsyncEmailResourceWithStreamingResponse",
    "ProfilesResource",
    "AsyncProfilesResource",
    "ProfilesResourceWithRawResponse",
    "AsyncProfilesResourceWithRawResponse",
    "ProfilesResourceWithStreamingResponse",
    "AsyncProfilesResourceWithStreamingResponse",
    "LimitsResource",
    "AsyncLimitsResource",
    "LimitsResourceWithRawResponse",
    "AsyncLimitsResourceWithRawResponse",
    "LimitsResourceWithStreamingResponse",
    "AsyncLimitsResourceWithStreamingResponse",
    "EntriesResource",
    "AsyncEntriesResource",
    "EntriesResourceWithRawResponse",
    "AsyncEntriesResourceWithRawResponse",
    "EntriesResourceWithStreamingResponse",
    "AsyncEntriesResourceWithStreamingResponse",
    "DLPResource",
    "AsyncDLPResource",
    "DLPResourceWithRawResponse",
    "AsyncDLPResourceWithRawResponse",
    "DLPResourceWithStreamingResponse",
    "AsyncDLPResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .subdomains import (
    SubdomainsResource,
    AsyncSubdomainsResource,
    SubdomainsResourceWithRawResponse,
    AsyncSubdomainsResourceWithRawResponse,
    SubdomainsResourceWithStreamingResponse,
    AsyncSubdomainsResourceWithStreamingResponse,
)
from .email_sending import (
    EmailSendingResource,
    AsyncEmailSendingResource,
    EmailSendingResourceWithRawResponse,
    AsyncEmailSendingResourceWithRawResponse,
    EmailSendingResourceWithStreamingResponse,
    AsyncEmailSendingResourceWithStreamingResponse,
)

__all__ = [
    "SubdomainsResource",
    "AsyncSubdomainsResource",
    "SubdomainsResourceWithRawResponse",
    "AsyncSubdomainsResourceWithRawResponse",
    "SubdomainsResourceWithStreamingResponse",
    "AsyncSubdomainsResourceWithStreamingResponse",
    "EmailSendingResource",
    "AsyncEmailSendingResource",
    "EmailSendingResourceWithRawResponse",
    "AsyncEmailSendingResourceWithRawResponse",
    "EmailSendingResourceWithStreamingResponse",
    "AsyncEmailSendingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict
from datetime import datetime
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel
from ..configuration import Configuration

__all__ = ["ConfigGetResponse", "ConfigGetResponseItem"]


class ConfigGetResponseItem(BaseModel):
    id: int
    """ID of the configuration."""

    config: Configuration
    """Zaraz configuration."""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Date and time the configuration was created."""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Date and time the configuration was last updated."""

    user_id: str = FieldInfo(alias="userId")
    """Alpha-numeric ID of the account user who published the configuration."""


ConfigGetResponse: TypeAlias = Dict[str, ConfigGetResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from .output_options import OutputOptions

__all__ = ["LogpushJob"]


class LogpushJob(BaseModel):
    id: Optional[int] = None
    """Unique id of the job."""

    dataset: Optional[
        Literal[
            "access_requests",
            "audit_logs",
            "audit_logs_v2",
            "biso_user_actions",
            "casb_findings",
            "device_posture_results",
            "dex_application_tests",
            "dex_device_state_events",
            "dlp_forensic_copies",
            "dns_firewall_logs",
            "dns_logs",
            "email_security_alerts",
            "email_security_post_delivery_events",
            "firewall_events",
            "gateway_dns",
            "gateway_http",
            "gateway_network",
            "http_requests",
            "ipsec_logs",
            "magic_ids_detections",
            "mcp_portal_logs",
            "mnm_flow_logs",
            "nel_reports",
            "network_analytics_logs",
            "page_shield_events",
            "sinkhole_http_logs",
            "spectrum_events",
            "ssh_logs",
            "warp_config_changes",
            "warp_toggle_changes",
            "workers_trace_events",
            "zaraz_events",
            "zero_trust_network_sessions",
        ]
    ] = None
    """Name of the dataset.

    A list of supported datasets can be found on the
    [Developer Docs](https://developers.cloudflare.com/logs/reference/log-fields/).
    """

    destination_conf: Optional[str] = None
    """Uniquely identifies a resource (such as an s3 bucket) where data.

    will be pushed. Additional configuration parameters supported by the destination
    may be included.
    """

    enabled: Optional[bool] = None
    """Flag that indicates if the job is enabled."""

    error_message: Optional[str] = None
    """If not null, the job is currently failing.

    Failures are usually. repetitive (example: no permissions to write to
    destination bucket). Only the last failure is recorded. On successful execution
    of a job the error_message and last_error are set to null.
    """

    frequency: Optional[Literal["high", "low"]] = None
    """This field is deprecated.

    Please use `max_upload_*` parameters instead. . The frequency at which
    Cloudflare sends batches of logs to your destination. Setting frequency to high
    sends your logs in larger quantities of smaller files. Setting frequency to low
    sends logs in smaller quantities of larger files.
    """

    kind: Optional[Literal["", "edge"]] = None
    """
    The kind parameter (optional) is used to differentiate between Logpush and Edge
    Log Delivery jobs (when supported by the dataset).
    """

    last_complete: Optional[datetime] = None
    """Records the last time for which logs have been successfully pushed.

    If the last successful push was for logs range 2018-07-23T10:00:00Z to
    2018-07-23T10:01:00Z then the value of this field will be 2018-07-23T10:01:00Z.
    If the job has never run or has just been enabled and hasn't run yet then the
    field will be empty.
    """

    last_error: Optional[datetime] = None
    """Records the last time the job failed.

    If not null, the job is currently. failing. If null, the job has either never
    failed or has run successfully at least once since last failure. See also the
    error_message field.
    """

    logpull_options: Optional[str] = None
    """This field is deprecated.

    Use `output_options` instead. Configuration string. It specifies things like
    requested fields and timestamp formats. If migrating from the logpull api, copy
    the url (full url or just the query string) of your call here, and logpush will
    keep on making this call for you, setting start and end times appropriately.
    """

    max_upload_bytes: Union[Literal[0], int, None] = None
    """The maximum uncompressed file size of a batch of logs.

    This setting value must be between `5 MB` and `1 GB`, or `0` to disable it. Note
    that you cannot set a minimum file size; this means that log files may be much
    smaller than this batch size.
    """

    max_upload_interval_seconds: Union[Literal[0], int, None] = None
    """The maximum interval in seconds for log batches.

    This setting must be between 30 and 300 seconds (5 minutes), or `0` to disable
    it. Note that you cannot specify a minimum interval for log batches; this means
    that log files may be sent in shorter intervals than this.
    """

    max_upload_records: Union[Literal[0], int, None] = None
    """The maximum number of log lines per batch.

    This setting must be between 1000 and 1,000,000 lines, or `0` to disable it.
    Note that you cannot specify a minimum number of log lines per batch; this means
    that log files may contain many fewer lines than this.
    """

    name: Optional[str] = None
    """Optional human readable job name.

    Not unique. Cloudflare suggests. that you set this to a meaningful string, like
    the domain name, to make it easier to identify your job.
    """

    output_options: Optional[OutputOptions] = None
    """The structured replacement for `logpull_options`.

    When including this field, the `logpull_option` field will be ignored.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .credential_edit_params import CredentialEditParams as CredentialEditParams
from .credential_list_params import CredentialListParams as CredentialListParams
from .credential_get_response import CredentialGetResponse as CredentialGetResponse
from .credential_create_params import CredentialCreateParams as CredentialCreateParams
from .credential_edit_response import CredentialEditResponse as CredentialEditResponse
from .credential_list_response import CredentialListResponse as CredentialListResponse
from .credential_update_params import CredentialUpdateParams as CredentialUpdateParams
from .credential_create_response import CredentialCreateResponse as CredentialCreateResponse
from .credential_update_response import CredentialUpdateResponse as CredentialUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = [
    "IndicatorFeedGetResponse",
    "LastUploadSummary",
    "LastUploadSummaryPersisted",
    "LastUploadSummarySkipped",
    "LastUploadSummaryUploaded",
]


class LastUploadSummaryPersisted(BaseModel):
    """Net delta applied to feed indicators by this upload.

    Snapshot
    uploads emit both *_added and *_removed; delta-add emits only
    *_added; delta-remove emits only *_removed.
    """

    domains_added: Optional[int] = None

    domains_removed: Optional[int] = None

    ips_added: Optional[int] = None

    ips_removed: Optional[int] = None

    urls_added: Optional[int] = None

    urls_removed: Optional[int] = None


class LastUploadSummarySkipped(BaseModel):
    """
    Counts of indicators that were uploaded but did not reach
    QuickSilver, broken down by reason.
    """

    allowlisted_domains: Optional[int] = None
    """Domains filtered by the global popularity allowlist at QS provisioning time.

    Popular domains (bing.com, naver.com, etc.) are protected from
    custom-threat-feed enforcement.
    """

    expired_indicators: Optional[int] = None
    """Indicators in the upload whose valid_until is already in the past.

    These are not added to QS; the expiration cron handles cleanup.
    """

    invalid_indicators: Optional[int] = None
    """Reserved for future use.

    Currently always 0 — the unifier aborts the entire upload on a single bad
    indicator.
    """


class LastUploadSummaryUploaded(BaseModel):
    """Indicator counts from the unified file the loader received"""

    domains: Optional[int] = None
    """Number of domain indicators in the upload"""

    ips: Optional[int] = None
    """Number of IP indicators in the upload"""

    urls: Optional[int] = None
    """Number of URL indicators in the upload"""


class LastUploadSummary(BaseModel):
    """Summary of indicator counts from the last successful upload to this
    feed.

    Populated by the custom-threat-feeds loader at the end of each
    successful load. Absent (omitted) when no upload has completed
    successfully or the upload errored before the summary write.
    Surfaces silent-failure paths so operators can see when their
    indicators were dropped (popularity allowlist, expired valid_until,
    etc.) without reading loader logs.
    """

    persisted: Optional[LastUploadSummaryPersisted] = None
    """Net delta applied to feed indicators by this upload.

    Snapshot uploads emit both _\\__added and _\\__removed; delta-add emits only
    _\\__added; delta-remove emits only _\\__removed.
    """

    skipped: Optional[LastUploadSummarySkipped] = None
    """
    Counts of indicators that were uploaded but did not reach QuickSilver, broken
    down by reason.
    """

    uploaded: Optional[LastUploadSummaryUploaded] = None
    """Indicator counts from the unified file the loader received"""


class IndicatorFeedGetResponse(BaseModel):
    id: Optional[int] = None
    """The unique identifier for the indicator feed"""

    created_on: Optional[datetime] = None
    """The date and time when the data entry was created"""

    description: Optional[str] = None
    """The description of the example test"""

    is_attributable: Optional[bool] = None
    """Whether the indicator feed can be attributed to a provider"""

    is_downloadable: Optional[bool] = None
    """Whether the indicator feed can be downloaded"""

    is_public: Optional[bool] = None
    """Whether the indicator feed is exposed to customers"""

    last_upload_summary: Optional[LastUploadSummary] = None
    """Summary of indicator counts from the last successful upload to this feed.

    Populated by the custom-threat-feeds loader at the end of each successful load.
    Absent (omitted) when no upload has completed successfully or the upload errored
    before the summary write. Surfaces silent-failure paths so operators can see
    when their indicators were dropped (popularity allowlist, expired valid_until,
    etc.) without reading loader logs.
    """

    latest_upload_error: Optional[str] = None
    """Human-readable error message describing why the latest upload failed.

    Populated only when `latest_upload_status` is `Error`. Returns one of a small
    fixed set of category-level messages (invalid domain / IP / URL entries,
    malformed row or header, invalid valid_until timestamp, etc.) or the generic
    `Upload failed` for unknown or infrastructure-level errors. Never echoes raw
    error text from the underlying loader. Intel accounts receive the verbatim
    loader/API error text (including specific offending values) instead of these
    category-level messages.
    """

    latest_upload_status: Optional[Literal["Mirroring", "Unifying", "Loading", "Provisioning", "Complete", "Error"]] = (
        None
    )
    """Status of the latest snapshot uploaded"""

    modified_on: Optional[datetime] = None
    """The date and time when the data entry was last modified"""

    name: Optional[str] = None
    """The name of the indicator feed"""

    provider_id: Optional[int] = None
    """The unique identifier for the provider"""

    provider_name: Optional[str] = None
    """The provider of the indicator feed"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ScheduleDeleteResponse"]


class ScheduleDeleteResponse(BaseModel):
    count: Optional[float] = None
    """Number of items affected."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["CustomPage"]


class CustomPage(BaseModel):
    custom_html: str
    """Custom page HTML."""

    name: str
    """Custom page name."""

    type: Literal["identity_denied", "forbidden"]
    """Custom page type."""

    uid: Optional[str] = None
    """UUID."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["Key"]


class Key(BaseModel):
    """A name for a value.

    A value stored under a given key may be retrieved via the same key.
    """

    name: str
    """A key's name.

    The name may be at most 512 bytes. All printable, non-whitespace characters are
    valid. Use percent-encoding to define key names as part of a URL.
    """

    expiration: Optional[float] = None
    """
    The time, measured in number of seconds since the UNIX epoch, at which the key
    will expire. This property is omitted for keys that will not expire.
    """

    metadata: Optional[object] = None
    """Arbitrary JSON that is associated with a key."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .routes import (
    RoutesResource,
    AsyncRoutesResource,
    RoutesResourceWithRawResponse,
    AsyncRoutesResourceWithRawResponse,
    RoutesResourceWithStreamingResponse,
    AsyncRoutesResourceWithStreamingResponse,
)
from .subnets import (
    SubnetsResource,
    AsyncSubnetsResource,
    SubnetsResourceWithRawResponse,
    AsyncSubnetsResourceWithRawResponse,
    SubnetsResourceWithStreamingResponse,
    AsyncSubnetsResourceWithStreamingResponse,
)
from .networks import (
    NetworksResource,
    AsyncNetworksResource,
    NetworksResourceWithRawResponse,
    AsyncNetworksResourceWithRawResponse,
    NetworksResourceWithStreamingResponse,
    AsyncNetworksResourceWithStreamingResponse,
)
from .hostname_routes import (
    HostnameRoutesResource,
    AsyncHostnameRoutesResource,
    HostnameRoutesResourceWithRawResponse,
    AsyncHostnameRoutesResourceWithRawResponse,
    HostnameRoutesResourceWithStreamingResponse,
    AsyncHostnameRoutesResourceWithStreamingResponse,
)
from .virtual_networks import (
    VirtualNetworksResource,
    AsyncVirtualNetworksResource,
    VirtualNetworksResourceWithRawResponse,
    AsyncVirtualNetworksResourceWithRawResponse,
    VirtualNetworksResourceWithStreamingResponse,
    AsyncVirtualNetworksResourceWithStreamingResponse,
)

__all__ = [
    "RoutesResource",
    "AsyncRoutesResource",
    "RoutesResourceWithRawResponse",
    "AsyncRoutesResourceWithRawResponse",
    "RoutesResourceWithStreamingResponse",
    "AsyncRoutesResourceWithStreamingResponse",
    "VirtualNetworksResource",
    "AsyncVirtualNetworksResource",
    "VirtualNetworksResourceWithRawResponse",
    "AsyncVirtualNetworksResourceWithRawResponse",
    "VirtualNetworksResourceWithStreamingResponse",
    "AsyncVirtualNetworksResourceWithStreamingResponse",
    "SubnetsResource",
    "AsyncSubnetsResource",
    "SubnetsResourceWithRawResponse",
    "AsyncSubnetsResourceWithRawResponse",
    "SubnetsResourceWithStreamingResponse",
    "AsyncSubnetsResourceWithStreamingResponse",
    "HostnameRoutesResource",
    "AsyncHostnameRoutesResource",
    "HostnameRoutesResourceWithRawResponse",
    "AsyncHostnameRoutesResourceWithRawResponse",
    "HostnameRoutesResourceWithStreamingResponse",
    "AsyncHostnameRoutesResourceWithStreamingResponse",
    "NetworksResource",
    "AsyncNetworksResource",
    "NetworksResourceWithRawResponse",
    "AsyncNetworksResourceWithRawResponse",
    "NetworksResourceWithStreamingResponse",
    "AsyncNetworksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .seats import (
    SeatsResource,
    AsyncSeatsResource,
    SeatsResourceWithRawResponse,
    AsyncSeatsResourceWithRawResponse,
    SeatsResourceWithStreamingResponse,
    AsyncSeatsResourceWithStreamingResponse,
)
from .dex.dex import (
    DEXResource,
    AsyncDEXResource,
    DEXResourceWithRawResponse,
    AsyncDEXResourceWithRawResponse,
    DEXResourceWithStreamingResponse,
    AsyncDEXResourceWithStreamingResponse,
)
from .dlp.dlp import (
    DLPResource,
    AsyncDLPResource,
    DLPResourceWithRawResponse,
    AsyncDLPResourceWithRawResponse,
    DLPResourceWithStreamingResponse,
    AsyncDLPResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .access.access import (
    AccessResource,
    AsyncAccessResource,
    AccessResourceWithRawResponse,
    AsyncAccessResourceWithRawResponse,
    AccessResourceWithStreamingResponse,
    AsyncAccessResourceWithStreamingResponse,
)
from .devices.devices import (
    DevicesResource,
    AsyncDevicesResource,
    DevicesResourceWithRawResponse,
    AsyncDevicesResourceWithRawResponse,
    DevicesResourceWithStreamingResponse,
    AsyncDevicesResourceWithStreamingResponse,
)
from .gateway.gateway import (
    GatewayResource,
    AsyncGatewayResource,
    GatewayResourceWithRawResponse,
    AsyncGatewayResourceWithRawResponse,
    GatewayResourceWithStreamingResponse,
    AsyncGatewayResourceWithStreamingResponse,
)
from .tunnels.tunnels import (
    TunnelsResource,
    AsyncTunnelsResource,
    TunnelsResourceWithRawResponse,
    AsyncTunnelsResourceWithRawResponse,
    TunnelsResourceWithStreamingResponse,
    AsyncTunnelsResourceWithStreamingResponse,
)
from .networks.networks import (
    NetworksResource,
    AsyncNetworksResource,
    NetworksResourceWithRawResponse,
    AsyncNetworksResourceWithRawResponse,
    NetworksResourceWithStreamingResponse,
    AsyncNetworksResourceWithStreamingResponse,
)
from .connectivity_settings import (
    ConnectivitySettingsResource,
    AsyncConnectivitySettingsResource,
    ConnectivitySettingsResourceWithRawResponse,
    AsyncConnectivitySettingsResourceWithRawResponse,
    ConnectivitySettingsResourceWithStreamingResponse,
    AsyncConnectivitySettingsResourceWithStreamingResponse,
)
from .risk_scoring.risk_scoring import (
    RiskScoringResource,
    AsyncRiskScoringResource,
    RiskScoringResourceWithRawResponse,
    AsyncRiskScoringResourceWithRawResponse,
    RiskScoringResourceWithStreamingResponse,
    AsyncRiskScoringResourceWithStreamingResponse,
)
from .organizations.organizations import (
    OrganizationsResource,
    AsyncOrganizationsResource,
    OrganizationsResourceWithRawResponse,
    AsyncOrganizationsResourceWithRawResponse,
    OrganizationsResourceWithStreamingResponse,
    AsyncOrganizationsResourceWithStreamingResponse,
)
from .resource_library.resource_library import (
    ResourceLibraryResource,
    AsyncResourceLibraryResource,
    ResourceLibraryResourceWithRawResponse,
    AsyncResourceLibraryResourceWithRawResponse,
    ResourceLibraryResourceWithStreamingResponse,
    AsyncResourceLibraryResourceWithStreamingResponse,
)
from .identity_providers.identity_providers import (
    IdentityProvidersResource,
    AsyncIdentityProvidersResource,
    IdentityProvidersResourceWithRawResponse,
    AsyncIdentityProvidersResourceWithRawResponse,
    IdentityProvidersResourceWithStreamingResponse,
    AsyncIdentityProvidersResourceWithStreamingResponse,
)

__all__ = ["ZeroTrustResource", "AsyncZeroTrustResource"]


class ZeroTrustResource(SyncAPIResource):
    @cached_property
    def devices(self) -> DevicesResource:
        return DevicesResource(self._client)

    @cached_property
    def identity_providers(self) -> IdentityProvidersResource:
        return IdentityProvidersResource(self._client)

    @cached_property
    def organizations(self) -> OrganizationsResource:
        return OrganizationsResource(self._client)

    @cached_property
    def seats(self) -> SeatsResource:
        return SeatsResource(self._client)

    @cached_property
    def access(self) -> AccessResource:
        return AccessResource(self._client)

    @cached_property
    def dex(self) -> DEXResource:
        return DEXResource(self._client)

    @cached_property
    def tunnels(self) -> TunnelsResource:
        return TunnelsResource(self._client)

    @cached_property
    def connectivity_settings(self) -> ConnectivitySettingsResource:
        return ConnectivitySettingsResource(self._client)

    @cached_property
    def dlp(self) -> DLPResource:
        return DLPResource(self._client)

    @cached_property
    def gateway(self) -> GatewayResource:
        return GatewayResource(self._client)

    @cached_property
    def networks(self) -> NetworksResource:
        return NetworksResource(self._client)

    @cached_property
    def risk_scoring(self) -> RiskScoringResource:
        return RiskScoringResource(self._client)

    @cached_property
    def resource_library(self) -> ResourceLibraryResource:
        return ResourceLibraryResource(self._client)

    @cached_property
    def with_raw_response(self) -> ZeroTrustResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ZeroTrustResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ZeroTrustResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ZeroTrustResourceWithStreamingResponse(self)


class AsyncZeroTrustResource(AsyncAPIResource):
    @cached_property
    def devices(self) -> AsyncDevicesResource:
        return AsyncDevicesResource(self._client)

    @cached_property
    def identity_providers(self) -> AsyncIdentityProvidersResource:
        return AsyncIdentityProvidersResource(self._client)

    @cached_property
    def organizations(self) -> AsyncOrganizationsResource:
        return AsyncOrganizationsResource(self._client)

    @cached_property
    def seats(self) -> AsyncSeatsResource:
        return AsyncSeatsResource(self._client)

    @cached_property
    def access(self) -> AsyncAccessResource:
        return AsyncAccessResource(self._client)

    @cached_property
    def dex(self) -> AsyncDEXResource:
        return AsyncDEXResource(self._client)

    @cached_property
    def tunnels(self) -> AsyncTunnelsResource:
        return AsyncTunnelsResource(self._client)

    @cached_property
    def connectivity_settings(self) -> AsyncConnectivitySettingsResource:
        return AsyncConnectivitySettingsResource(self._client)

    @cached_property
    def dlp(self) -> AsyncDLPResource:
        return AsyncDLPResource(self._client)

    @cached_property
    def gateway(self) -> AsyncGatewayResource:
        return AsyncGatewayResource(self._client)

    @cached_property
    def networks(self) -> AsyncNetworksResource:
        return AsyncNetworksResource(self._client)

    @cached_property
    def risk_scoring(self) -> AsyncRiskScoringResource:
        return AsyncRiskScoringResource(self._client)

    @cached_property
    def resource_library(self) -> AsyncResourceLibraryResource:
        return AsyncResourceLibraryResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncZeroTrustResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncZeroTrustResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncZeroTrustResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncZeroTrustResourceWithStreamingResponse(self)


class ZeroTrustResourceWithRawResponse:
    def __init__(self, zero_trust: ZeroTrustResource) -> None:
        self._zero_trust = zero_trust

    @cached_property
    def devices(self) -> DevicesResourceWithRawResponse:
        return DevicesResourceWithRawResponse(self._zero_trust.devices)

    @cached_property
    def identity_providers(self) -> IdentityProvidersResourceWithRawResponse:
        return IdentityProvidersResourceWithRawResponse(self._zero_trust.identity_providers)

    @cached_property
    def organizations(self) -> OrganizationsResourceWithRawResponse:
        return OrganizationsResourceWithRawResponse(self._zero_trust.organizations)

    @cached_property
    def seats(self) -> SeatsResourceWithRawResponse:
        return SeatsResourceWithRawResponse(self._zero_trust.seats)

    @cached_property
    def access(self) -> AccessResourceWithRawResponse:
        return AccessResourceWithRawResponse(self._zero_trust.access)

    @cached_property
    def dex(self) -> DEXResourceWithRawResponse:
        return DEXResourceWithRawResponse(self._zero_trust.dex)

    @cached_property
    def tunnels(self) -> TunnelsResourceWithRawResponse:
        return TunnelsResourceWithRawResponse(self._zero_trust.tunnels)

    @cached_property
    def connectivity_settings(self) -> ConnectivitySettingsResourceWithRawResponse:
        return ConnectivitySettingsResourceWithRawResponse(self._zero_trust.connectivity_settings)

    @cached_property
    def dlp(self) -> DLPResourceWithRawResponse:
        return DLPResourceWithRawResponse(self._zero_trust.dlp)

    @cached_property
    def gateway(self) -> GatewayResourceWithRawResponse:
        return GatewayResourceWithRawResponse(self._zero_trust.gateway)

    @cached_property
    def networks(self) -> NetworksResourceWithRawResponse:
        return NetworksResourceWithRawResponse(self._zero_trust.networks)

    @cached_property
    def risk_scoring(self) -> RiskScoringResourceWithRawResponse:
        return RiskScoringResourceWithRawResponse(self._zero_trust.risk_scoring)

    @cached_property
    def resource_library(self) -> ResourceLibraryResourceWithRawResponse:
        return ResourceLibraryResourceWithRawResponse(self._zero_trust.resource_library)


class AsyncZeroTrustResourceWithRawResponse:
    def __init__(self, zero_trust: AsyncZeroTrustResource) -> None:
        self._zero_trust = zero_trust

    @cached_property
    def devices(self) -> AsyncDevicesResourceWithRawResponse:
        return AsyncDevicesResourceWithRawResponse(self._zero_trust.devices)

    @cached_property
    def identity_providers(self) -> AsyncIdentityProvidersResourceWithRawResponse:
        return AsyncIdentityProvidersResourceWithRawResponse(self._zero_trust.identity_providers)

    @cached_property
    def organizations(self) -> AsyncOrganizationsResourceWithRawResponse:
        return AsyncOrganizationsResourceWithRawResponse(self._zero_trust.organizations)

    @cached_property
    def seats(self) -> AsyncSeatsResourceWithRawResponse:
        return AsyncSeatsResourceWithRawResponse(self._zero_trust.seats)

    @cached_property
    def access(self) -> AsyncAccessResourceWithRawResponse:
        return AsyncAccessResourceWithRawResponse(self._zero_trust.access)

    @cached_property
    def dex(self) -> AsyncDEXResourceWithRawResponse:
        return AsyncDEXResourceWithRawResponse(self._zero_trust.dex)

    @cached_property
    def tunnels(self) -> AsyncTunnelsResourceWithRawResponse:
        return AsyncTunnelsResourceWithRawResponse(self._zero_trust.tunnels)

    @cached_property
    def connectivity_settings(self) -> AsyncConnectivitySettingsResourceWithRawResponse:
        return AsyncConnectivitySettingsResourceWithRawResponse(self._zero_trust.connectivity_settings)

    @cached_property
    def dlp(self) -> AsyncDLPResourceWithRawResponse:
        return AsyncDLPResourceWithRawResponse(self._zero_trust.dlp)

    @cached_property
    def gateway(self) -> AsyncGatewayResourceWithRawResponse:
        return AsyncGatewayResourceWithRawResponse(self._zero_trust.gateway)

    @cached_property
    def networks(self) -> AsyncNetworksResourceWithRawResponse:
        return AsyncNetworksResourceWithRawResponse(self._zero_trust.networks)

    @cached_property
    def risk_scoring(self) -> AsyncRiskScoringResourceWithRawResponse:
        return AsyncRiskScoringResourceWithRawResponse(self._zero_trust.risk_scoring)

    @cached_property
    def resource_library(self) -> AsyncResourceLibraryResourceWithRawResponse:
        return AsyncResourceLibraryResourceWithRawResponse(self._zero_trust.resource_library)


class ZeroTrustResourceWithStreamingResponse:
    def __init__(self, zero_trust: ZeroTrustResource) -> None:
        self._zero_trust = zero_trust

    @cached_property
    def devices(self) -> DevicesResourceWithStreamingResponse:
        return DevicesResourceWithStreamingResponse(self._zero_trust.devices)

    @cached_property
    def identity_providers(self) -> IdentityProvidersResourceWithStreamingResponse:
        return IdentityProvidersResourceWithStreamingResponse(self._zero_trust.identity_providers)

    @cached_property
    def organizations(self) -> OrganizationsResourceWithStreamingResponse:
        return OrganizationsResourceWithStreamingResponse(self._zero_trust.organizations)

    @cached_property
    def seats(self) -> SeatsResourceWithStreamingResponse:
        return SeatsResourceWithStreamingResponse(self._zero_trust.seats)

    @cached_property
    def access(self) -> AccessResourceWithStreamingResponse:
        return AccessResourceWithStreamingResponse(self._zero_trust.access)

    @cached_property
    def dex(self) -> DEXResourceWithStreamingResponse:
        return DEXResourceWithStreamingResponse(self._zero_trust.dex)

    @cached_property
    def tunnels(self) -> TunnelsResourceWithStreamingResponse:
        return TunnelsResourceWithStreamingResponse(self._zero_trust.tunnels)

    @cached_property
    def connectivity_settings(self) -> ConnectivitySettingsResourceWithStreamingResponse:
        return ConnectivitySettingsResourceWithStreamingResponse(self._zero_trust.connectivity_settings)

    @cached_property
    def dlp(self) -> DLPResourceWithStreamingResponse:
        return DLPResourceWithStreamingResponse(self._zero_trust.dlp)

    @cached_property
    def gateway(self) -> GatewayResourceWithStreamingResponse:
        return GatewayResourceWithStreamingResponse(self._zero_trust.gateway)

    @cached_property
    def networks(self) -> NetworksResourceWithStreamingResponse:
        return NetworksResourceWithStreamingResponse(self._zero_trust.networks)

    @cached_property
    def risk_scoring(self) -> RiskScoringResourceWithStreamingResponse:
        return RiskScoringResourceWithStreamingResponse(self._zero_trust.risk_scoring)

    @cached_property
    def resource_library(self) -> ResourceLibraryResourceWithStreamingResponse:
        return ResourceLibraryResourceWithStreamingResponse(self._zero_trust.resource_library)


class AsyncZeroTrustResourceWithStreamingResponse:
    def __init__(self, zero_trust: AsyncZeroTrustResource) -> None:
        self._zero_trust = zero_trust

    @cached_property
    def devices(self) -> AsyncDevicesResourceWithStreamingResponse:
        return AsyncDevicesResourceWithStreamingResponse(self._zero_trust.devices)

    @cached_property
    def identity_providers(self) -> AsyncIdentityProvidersResourceWithStreamingResponse:
        return AsyncIdentityProvidersResourceWithStreamingResponse(self._zero_trust.identity_providers)

    @cached_property
    def organizations(self) -> AsyncOrganizationsResourceWithStreamingResponse:
        return AsyncOrganizationsResourceWithStreamingResponse(self._zero_trust.organizations)

    @cached_property
    def seats(self) -> AsyncSeatsResourceWithStreamingResponse:
        return AsyncSeatsResourceWithStreamingResponse(self._zero_trust.seats)

    @cached_property
    def access(self) -> AsyncAccessResourceWithStreamingResponse:
        return AsyncAccessResourceWithStreamingResponse(self._zero_trust.access)

    @cached_property
    def dex(self) -> AsyncDEXResourceWithStreamingResponse:
        return AsyncDEXResourceWithStreamingResponse(self._zero_trust.dex)

    @cached_property
    def tunnels(self) -> AsyncTunnelsResourceWithStreamingResponse:
        return AsyncTunnelsResourceWithStreamingResponse(self._zero_trust.tunnels)

    @cached_property
    def connectivity_settings(self) -> AsyncConnectivitySettingsResourceWithStreamingResponse:
        return AsyncConnectivitySettingsResourceWithStreamingResponse(self._zero_trust.connectivity_settings)

    @cached_property
    def dlp(self) -> AsyncDLPResourceWithStreamingResponse:
        return AsyncDLPResourceWithStreamingResponse(self._zero_trust.dlp)

    @cached_property
    def gateway(self) -> AsyncGatewayResourceWithStreamingResponse:
        return AsyncGatewayResourceWithStreamingResponse(self._zero_trust.gateway)

    @cached_property
    def networks(self) -> AsyncNetworksResourceWithStreamingResponse:
        return AsyncNetworksResourceWithStreamingResponse(self._zero_trust.networks)

    @cached_property
    def risk_scoring(self) -> AsyncRiskScoringResourceWithStreamingResponse:
        return AsyncRiskScoringResourceWithStreamingResponse(self._zero_trust.risk_scoring)

    @cached_property
    def resource_library(self) -> AsyncResourceLibraryResourceWithStreamingResponse:
        return AsyncResourceLibraryResourceWithStreamingResponse(self._zero_trust.resource_library)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["SlotListResponse", "Item", "ItemFacility"]


class ItemFacility(BaseModel):
    address: List[str]

    name: str


class Item(BaseModel):
    id: str
    """Slot ID"""

    facility: ItemFacility

    occupied: bool
    """Whether the slot is occupied or not"""

    site: str

    speed: str

    account: Optional[str] = None
    """Customer account tag"""


class SlotListResponse(BaseModel):
    items: List[Item]

    next: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["SessionGetSessionParticipantsResponse", "Data", "DataParticipant"]


class DataParticipant(BaseModel):
    id: Optional[str] = None
    """Participant ID. This maps to the corresponding peerId."""

    created_at: Optional[str] = None
    """timestamp when this participant was created."""

    custom_participant_id: Optional[str] = None
    """ID passed by client to create this participant."""

    display_name: Optional[str] = None
    """Display name of participant when joining the session."""

    duration: Optional[float] = None
    """number of minutes for which the participant was in the session."""

    joined_at: Optional[str] = None
    """timestamp at which participant joined the session."""

    left_at: Optional[str] = None
    """timestamp at which participant left the session."""

    preset_name: Optional[str] = None
    """Name of the preset associated with the participant."""

    updated_at: Optional[str] = None
    """timestamp when this participant's data was last updated."""

    user_id: Optional[str] = None
    """User id for this participant."""


class Data(BaseModel):
    participants: Optional[List[DataParticipant]] = None


class SessionGetSessionParticipantsResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .setting import Setting as Setting
from .setting_value import SettingValue as SettingValue
from .tls_get_response import TLSGetResponse as TLSGetResponse
from .tls_update_params import TLSUpdateParams as TLSUpdateParams
from .setting_value_param import SettingValueParam as SettingValueParam
from .tls_delete_response import TLSDeleteResponse as TLSDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ObjectDeleteResponse"]


class ObjectDeleteResponse(BaseModel):
    """Result of a successful object deletion."""

    key: Optional[str] = None
    """The key (name) of the deleted object."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SSOUpdateResponse", "Verification"]


class Verification(BaseModel):
    code: Optional[str] = None
    """DNS verification code.

    Add this entire string to the DNS TXT record of the email domain to validate
    ownership.
    """

    status: Optional[Literal["awaiting", "pending", "failed", "verified"]] = None
    """The status of the verification code from the verification process."""


class SSOUpdateResponse(BaseModel):
    id: Optional[str] = None
    """SSO Connector identifier tag."""

    created_on: Optional[datetime] = None
    """Timestamp for the creation of the SSO connector"""

    email_domain: Optional[str] = None

    enabled: Optional[bool] = None

    updated_on: Optional[datetime] = None
    """Timestamp for the last update of the SSO connector"""

    use_fedramp_language: Optional[bool] = None
    """Controls the display of FedRAMP language to the user during SSO login"""

    verification: Optional[Verification] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import TYPE_CHECKING, List
from typing_extensions import TypeAlias, TypeAliasType

from ..._compat import PYDANTIC_V1

__all__ = ["Trace"]

if TYPE_CHECKING or not PYDANTIC_V1:
    Trace = TypeAliasType("Trace", List["TraceItem"])
else:
    Trace: TypeAlias = List["TraceItem"]

from .trace_item import TraceItem



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .nat_param import NatParam

__all__ = ["RoutedSubnetParam"]


class RoutedSubnetParam(TypedDict, total=False):
    next_hop: Required[str]
    """A valid IPv4 address."""

    prefix: Required[str]
    """A valid CIDR notation representing an IP range."""

    nat: NatParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.queues import purge_start_params
from ...types.queues.queue import Queue
from ...types.queues.purge_status_response import PurgeStatusResponse

__all__ = ["PurgeResource", "AsyncPurgeResource"]


class PurgeResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PurgeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PurgeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PurgeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PurgeResourceWithStreamingResponse(self)

    def start(
        self,
        queue_id: str,
        *,
        account_id: str,
        delete_messages_permanently: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Queue]:
        """
        Deletes all messages from the Queue.

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          delete_messages_permanently: Confimation that all messages will be deleted permanently.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/queues/{queue_id}/purge", account_id=account_id, queue_id=queue_id),
            body=maybe_transform(
                {"delete_messages_permanently": delete_messages_permanently}, purge_start_params.PurgeStartParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Queue]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Queue]], ResultWrapper[Queue]),
        )

    def status(
        self,
        queue_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PurgeStatusResponse]:
        """
        Get details about a Queue's purge status.

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/queues/{queue_id}/purge", account_id=account_id, queue_id=queue_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PurgeStatusResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PurgeStatusResponse]], ResultWrapper[PurgeStatusResponse]),
        )


class AsyncPurgeResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPurgeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPurgeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPurgeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPurgeResourceWithStreamingResponse(self)

    async def start(
        self,
        queue_id: str,
        *,
        account_id: str,
        delete_messages_permanently: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Queue]:
        """
        Deletes all messages from the Queue.

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          delete_messages_permanently: Confimation that all messages will be deleted permanently.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/queues/{queue_id}/purge", account_id=account_id, queue_id=queue_id),
            body=await async_maybe_transform(
                {"delete_messages_permanently": delete_messages_permanently}, purge_start_params.PurgeStartParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Queue]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Queue]], ResultWrapper[Queue]),
        )

    async def status(
        self,
        queue_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[PurgeStatusResponse]:
        """
        Get details about a Queue's purge status.

        Args:
          account_id: A Resource identifier.

          queue_id: A Resource identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not queue_id:
            raise ValueError(f"Expected a non-empty value for `queue_id` but received {queue_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/queues/{queue_id}/purge", account_id=account_id, queue_id=queue_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[PurgeStatusResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[PurgeStatusResponse]], ResultWrapper[PurgeStatusResponse]),
        )


class PurgeResourceWithRawResponse:
    def __init__(self, purge: PurgeResource) -> None:
        self._purge = purge

        self.start = to_raw_response_wrapper(
            purge.start,
        )
        self.status = to_raw_response_wrapper(
            purge.status,
        )


class AsyncPurgeResourceWithRawResponse:
    def __init__(self, purge: AsyncPurgeResource) -> None:
        self._purge = purge

        self.start = async_to_raw_response_wrapper(
            purge.start,
        )
        self.status = async_to_raw_response_wrapper(
            purge.status,
        )


class PurgeResourceWithStreamingResponse:
    def __init__(self, purge: PurgeResource) -> None:
        self._purge = purge

        self.start = to_streamed_response_wrapper(
            purge.start,
        )
        self.status = to_streamed_response_wrapper(
            purge.status,
        )


class AsyncPurgeResourceWithStreamingResponse:
    def __init__(self, purge: AsyncPurgeResource) -> None:
        self._purge = purge

        self.start = async_to_streamed_response_wrapper(
            purge.start,
        )
        self.status = async_to_streamed_response_wrapper(
            purge.status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from .events.events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from ...._base_client import AsyncPaginator, make_request_options
from .snapshots.snapshots import (
    SnapshotsResource,
    AsyncSnapshotsResource,
    SnapshotsResourceWithRawResponse,
    AsyncSnapshotsResourceWithRawResponse,
    SnapshotsResourceWithStreamingResponse,
    AsyncSnapshotsResourceWithStreamingResponse,
)
from ....types.magic_transit import (
    connector_edit_params,
    connector_list_params,
    connector_create_params,
    connector_update_params,
)
from ....types.magic_transit.connector_get_response import ConnectorGetResponse
from ....types.magic_transit.connector_edit_response import ConnectorEditResponse
from ....types.magic_transit.connector_list_response import ConnectorListResponse
from ....types.magic_transit.connector_create_response import ConnectorCreateResponse
from ....types.magic_transit.connector_delete_response import ConnectorDeleteResponse
from ....types.magic_transit.connector_update_response import ConnectorUpdateResponse

__all__ = ["ConnectorsResource", "AsyncConnectorsResource"]


class ConnectorsResource(SyncAPIResource):
    @cached_property
    def events(self) -> EventsResource:
        return EventsResource(self._client)

    @cached_property
    def snapshots(self) -> SnapshotsResource:
        return SnapshotsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConnectorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ConnectorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConnectorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ConnectorsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        device: connector_create_params.Device,
        activated: bool | Omit = omit,
        interrupt_window_days_of_week: List[
            Literal["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
        ]
        | Omit = omit,
        interrupt_window_duration_hours: float | Omit = omit,
        interrupt_window_embargo_dates: SequenceNotStr[str] | Omit = omit,
        interrupt_window_hour_of_day: float | Omit = omit,
        notes: str | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorCreateResponse:
        """
        Add a connector to your account

        Args:
          account_id: Account identifier

          device: Exactly one of id, serial_number, or provision_license must be provided.

          interrupt_window_days_of_week: Allowed days of the week for upgrades. Default is all days.

          interrupt_window_embargo_dates: List of dates (YYYY-MM-DD) when upgrades are blocked.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/magic/connectors", account_id=account_id),
            body=maybe_transform(
                {
                    "device": device,
                    "activated": activated,
                    "interrupt_window_days_of_week": interrupt_window_days_of_week,
                    "interrupt_window_duration_hours": interrupt_window_duration_hours,
                    "interrupt_window_embargo_dates": interrupt_window_embargo_dates,
                    "interrupt_window_hour_of_day": interrupt_window_hour_of_day,
                    "notes": notes,
                    "timezone": timezone,
                },
                connector_create_params.ConnectorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorCreateResponse], ResultWrapper[ConnectorCreateResponse]),
        )

    def update(
        self,
        connector_id: str,
        *,
        account_id: str,
        activated: bool | Omit = omit,
        interrupt_window_days_of_week: List[
            Literal["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
        ]
        | Omit = omit,
        interrupt_window_duration_hours: float | Omit = omit,
        interrupt_window_embargo_dates: SequenceNotStr[str] | Omit = omit,
        interrupt_window_hour_of_day: float | Omit = omit,
        notes: str | Omit = omit,
        provision_license: bool | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorUpdateResponse:
        """
        Replace Connector or Re-provision License Key

        Args:
          account_id: Account identifier

          interrupt_window_days_of_week: Allowed days of the week for upgrades. Default is all days.

          interrupt_window_embargo_dates: List of dates (YYYY-MM-DD) when upgrades are blocked.

          provision_license: When true, regenerate license key for the connector.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}",
                account_id=account_id,
                connector_id=connector_id,
            ),
            body=maybe_transform(
                {
                    "activated": activated,
                    "interrupt_window_days_of_week": interrupt_window_days_of_week,
                    "interrupt_window_duration_hours": interrupt_window_duration_hours,
                    "interrupt_window_embargo_dates": interrupt_window_embargo_dates,
                    "interrupt_window_hour_of_day": interrupt_window_hour_of_day,
                    "notes": notes,
                    "provision_license": provision_license,
                    "timezone": timezone,
                },
                connector_update_params.ConnectorUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorUpdateResponse], ResultWrapper[ConnectorUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        device_type: Literal["MANAGED", "LICENSED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[ConnectorListResponse]:
        """
        List Connectors

        Args:
          account_id: Account identifier

          device_type: Filter connectors by device type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/connectors", account_id=account_id),
            page=SyncSinglePage[ConnectorListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"device_type": device_type}, connector_list_params.ConnectorListParams),
            ),
            model=ConnectorListResponse,
        )

    def delete(
        self,
        connector_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorDeleteResponse:
        """
        Remove a connector from your account

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}",
                account_id=account_id,
                connector_id=connector_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorDeleteResponse], ResultWrapper[ConnectorDeleteResponse]),
        )

    def edit(
        self,
        connector_id: str,
        *,
        account_id: str,
        activated: bool | Omit = omit,
        interrupt_window_days_of_week: List[
            Literal["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
        ]
        | Omit = omit,
        interrupt_window_duration_hours: float | Omit = omit,
        interrupt_window_embargo_dates: SequenceNotStr[str] | Omit = omit,
        interrupt_window_hour_of_day: float | Omit = omit,
        notes: str | Omit = omit,
        provision_license: bool | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorEditResponse:
        """
        Edit Connector to update specific properties or Re-provision License Key

        Args:
          account_id: Account identifier

          interrupt_window_days_of_week: Allowed days of the week for upgrades. Default is all days.

          interrupt_window_embargo_dates: List of dates (YYYY-MM-DD) when upgrades are blocked.

          provision_license: When true, regenerate license key for the connector.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}",
                account_id=account_id,
                connector_id=connector_id,
            ),
            body=maybe_transform(
                {
                    "activated": activated,
                    "interrupt_window_days_of_week": interrupt_window_days_of_week,
                    "interrupt_window_duration_hours": interrupt_window_duration_hours,
                    "interrupt_window_embargo_dates": interrupt_window_embargo_dates,
                    "interrupt_window_hour_of_day": interrupt_window_hour_of_day,
                    "notes": notes,
                    "provision_license": provision_license,
                    "timezone": timezone,
                },
                connector_edit_params.ConnectorEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorEditResponse], ResultWrapper[ConnectorEditResponse]),
        )

    def get(
        self,
        connector_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorGetResponse:
        """
        Fetch Connector

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}",
                account_id=account_id,
                connector_id=connector_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorGetResponse], ResultWrapper[ConnectorGetResponse]),
        )


class AsyncConnectorsResource(AsyncAPIResource):
    @cached_property
    def events(self) -> AsyncEventsResource:
        return AsyncEventsResource(self._client)

    @cached_property
    def snapshots(self) -> AsyncSnapshotsResource:
        return AsyncSnapshotsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConnectorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConnectorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConnectorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncConnectorsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        device: connector_create_params.Device,
        activated: bool | Omit = omit,
        interrupt_window_days_of_week: List[
            Literal["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
        ]
        | Omit = omit,
        interrupt_window_duration_hours: float | Omit = omit,
        interrupt_window_embargo_dates: SequenceNotStr[str] | Omit = omit,
        interrupt_window_hour_of_day: float | Omit = omit,
        notes: str | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorCreateResponse:
        """
        Add a connector to your account

        Args:
          account_id: Account identifier

          device: Exactly one of id, serial_number, or provision_license must be provided.

          interrupt_window_days_of_week: Allowed days of the week for upgrades. Default is all days.

          interrupt_window_embargo_dates: List of dates (YYYY-MM-DD) when upgrades are blocked.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/magic/connectors", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "device": device,
                    "activated": activated,
                    "interrupt_window_days_of_week": interrupt_window_days_of_week,
                    "interrupt_window_duration_hours": interrupt_window_duration_hours,
                    "interrupt_window_embargo_dates": interrupt_window_embargo_dates,
                    "interrupt_window_hour_of_day": interrupt_window_hour_of_day,
                    "notes": notes,
                    "timezone": timezone,
                },
                connector_create_params.ConnectorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorCreateResponse], ResultWrapper[ConnectorCreateResponse]),
        )

    async def update(
        self,
        connector_id: str,
        *,
        account_id: str,
        activated: bool | Omit = omit,
        interrupt_window_days_of_week: List[
            Literal["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
        ]
        | Omit = omit,
        interrupt_window_duration_hours: float | Omit = omit,
        interrupt_window_embargo_dates: SequenceNotStr[str] | Omit = omit,
        interrupt_window_hour_of_day: float | Omit = omit,
        notes: str | Omit = omit,
        provision_license: bool | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorUpdateResponse:
        """
        Replace Connector or Re-provision License Key

        Args:
          account_id: Account identifier

          interrupt_window_days_of_week: Allowed days of the week for upgrades. Default is all days.

          interrupt_window_embargo_dates: List of dates (YYYY-MM-DD) when upgrades are blocked.

          provision_license: When true, regenerate license key for the connector.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}",
                account_id=account_id,
                connector_id=connector_id,
            ),
            body=await async_maybe_transform(
                {
                    "activated": activated,
                    "interrupt_window_days_of_week": interrupt_window_days_of_week,
                    "interrupt_window_duration_hours": interrupt_window_duration_hours,
                    "interrupt_window_embargo_dates": interrupt_window_embargo_dates,
                    "interrupt_window_hour_of_day": interrupt_window_hour_of_day,
                    "notes": notes,
                    "provision_license": provision_license,
                    "timezone": timezone,
                },
                connector_update_params.ConnectorUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorUpdateResponse], ResultWrapper[ConnectorUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        device_type: Literal["MANAGED", "LICENSED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ConnectorListResponse, AsyncSinglePage[ConnectorListResponse]]:
        """
        List Connectors

        Args:
          account_id: Account identifier

          device_type: Filter connectors by device type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/connectors", account_id=account_id),
            page=AsyncSinglePage[ConnectorListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"device_type": device_type}, connector_list_params.ConnectorListParams),
            ),
            model=ConnectorListResponse,
        )

    async def delete(
        self,
        connector_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorDeleteResponse:
        """
        Remove a connector from your account

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}",
                account_id=account_id,
                connector_id=connector_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorDeleteResponse], ResultWrapper[ConnectorDeleteResponse]),
        )

    async def edit(
        self,
        connector_id: str,
        *,
        account_id: str,
        activated: bool | Omit = omit,
        interrupt_window_days_of_week: List[
            Literal["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
        ]
        | Omit = omit,
        interrupt_window_duration_hours: float | Omit = omit,
        interrupt_window_embargo_dates: SequenceNotStr[str] | Omit = omit,
        interrupt_window_hour_of_day: float | Omit = omit,
        notes: str | Omit = omit,
        provision_license: bool | Omit = omit,
        timezone: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorEditResponse:
        """
        Edit Connector to update specific properties or Re-provision License Key

        Args:
          account_id: Account identifier

          interrupt_window_days_of_week: Allowed days of the week for upgrades. Default is all days.

          interrupt_window_embargo_dates: List of dates (YYYY-MM-DD) when upgrades are blocked.

          provision_license: When true, regenerate license key for the connector.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}",
                account_id=account_id,
                connector_id=connector_id,
            ),
            body=await async_maybe_transform(
                {
                    "activated": activated,
                    "interrupt_window_days_of_week": interrupt_window_days_of_week,
                    "interrupt_window_duration_hours": interrupt_window_duration_hours,
                    "interrupt_window_embargo_dates": interrupt_window_embargo_dates,
                    "interrupt_window_hour_of_day": interrupt_window_hour_of_day,
                    "notes": notes,
                    "provision_license": provision_license,
                    "timezone": timezone,
                },
                connector_edit_params.ConnectorEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorEditResponse], ResultWrapper[ConnectorEditResponse]),
        )

    async def get(
        self,
        connector_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectorGetResponse:
        """
        Fetch Connector

        Args:
          account_id: Account identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not connector_id:
            raise ValueError(f"Expected a non-empty value for `connector_id` but received {connector_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/magic/connectors/{connector_id}",
                account_id=account_id,
                connector_id=connector_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[ConnectorGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[ConnectorGetResponse], ResultWrapper[ConnectorGetResponse]),
        )


class ConnectorsResourceWithRawResponse:
    def __init__(self, connectors: ConnectorsResource) -> None:
        self._connectors = connectors

        self.create = to_raw_response_wrapper(
            connectors.create,
        )
        self.update = to_raw_response_wrapper(
            connectors.update,
        )
        self.list = to_raw_response_wrapper(
            connectors.list,
        )
        self.delete = to_raw_response_wrapper(
            connectors.delete,
        )
        self.edit = to_raw_response_wrapper(
            connectors.edit,
        )
        self.get = to_raw_response_wrapper(
            connectors.get,
        )

    @cached_property
    def events(self) -> EventsResourceWithRawResponse:
        return EventsResourceWithRawResponse(self._connectors.events)

    @cached_property
    def snapshots(self) -> SnapshotsResourceWithRawResponse:
        return SnapshotsResourceWithRawResponse(self._connectors.snapshots)


class AsyncConnectorsResourceWithRawResponse:
    def __init__(self, connectors: AsyncConnectorsResource) -> None:
        self._connectors = connectors

        self.create = async_to_raw_response_wrapper(
            connectors.create,
        )
        self.update = async_to_raw_response_wrapper(
            connectors.update,
        )
        self.list = async_to_raw_response_wrapper(
            connectors.list,
        )
        self.delete = async_to_raw_response_wrapper(
            connectors.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            connectors.edit,
        )
        self.get = async_to_raw_response_wrapper(
            connectors.get,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithRawResponse:
        return AsyncEventsResourceWithRawResponse(self._connectors.events)

    @cached_property
    def snapshots(self) -> AsyncSnapshotsResourceWithRawResponse:
        return AsyncSnapshotsResourceWithRawResponse(self._connectors.snapshots)


class ConnectorsResourceWithStreamingResponse:
    def __init__(self, connectors: ConnectorsResource) -> None:
        self._connectors = connectors

        self.create = to_streamed_response_wrapper(
            connectors.create,
        )
        self.update = to_streamed_response_wrapper(
            connectors.update,
        )
        self.list = to_streamed_response_wrapper(
            connectors.list,
        )
        self.delete = to_streamed_response_wrapper(
            connectors.delete,
        )
        self.edit = to_streamed_response_wrapper(
            connectors.edit,
        )
        self.get = to_streamed_response_wrapper(
            connectors.get,
        )

    @cached_property
    def events(self) -> EventsResourceWithStreamingResponse:
        return EventsResourceWithStreamingResponse(self._connectors.events)

    @cached_property
    def snapshots(self) -> SnapshotsResourceWithStreamingResponse:
        return SnapshotsResourceWithStreamingResponse(self._connectors.snapshots)


class AsyncConnectorsResourceWithStreamingResponse:
    def __init__(self, connectors: AsyncConnectorsResource) -> None:
        self._connectors = connectors

        self.create = async_to_streamed_response_wrapper(
            connectors.create,
        )
        self.update = async_to_streamed_response_wrapper(
            connectors.update,
        )
        self.list = async_to_streamed_response_wrapper(
            connectors.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            connectors.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            connectors.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            connectors.get,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithStreamingResponse:
        return AsyncEventsResourceWithStreamingResponse(self._connectors.events)

    @cached_property
    def snapshots(self) -> AsyncSnapshotsResourceWithStreamingResponse:
        return AsyncSnapshotsResourceWithStreamingResponse(self._connectors.snapshots)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["LiveInputCreateParams", "Recording"]


class LiveInputCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    default_creator: Annotated[str, PropertyInfo(alias="defaultCreator")]
    """Sets the creator ID asssociated with this live input."""

    delete_recording_after_days: Annotated[float, PropertyInfo(alias="deleteRecordingAfterDays")]
    """
    Indicates the number of days after which the live inputs recordings will be
    deleted. When a stream completes and the recording is ready, the value is used
    to calculate a scheduled deletion date for that recording. Omit the field to
    indicate no change, or include with a `null` value to remove an existing
    scheduled deletion.
    """

    enabled: bool
    """Indicates whether the live input is enabled and can accept streams."""

    meta: object
    """
    A user modifiable key-value store used to reference other systems of record for
    managing live inputs.
    """

    recording: Recording
    """Records the input to a Cloudflare Stream video.

    Behavior depends on the mode. In most cases, the video will initially be
    viewable as a live video and transition to on-demand after a condition is
    satisfied.
    """


class Recording(TypedDict, total=False):
    """Records the input to a Cloudflare Stream video.

    Behavior depends on the mode. In most cases, the video will initially be viewable as a live video and transition to on-demand after a condition is satisfied.
    """

    allowed_origins: Annotated[SequenceNotStr[str], PropertyInfo(alias="allowedOrigins")]
    """Lists the origins allowed to display videos created with this input.

    Enter allowed origin domains in an array and use `*` for wildcard subdomains. An
    empty array allows videos to be viewed on any origin.
    """

    hide_live_viewer_count: Annotated[bool, PropertyInfo(alias="hideLiveViewerCount")]
    """
    Disables reporting the number of live viewers when this property is set to
    `true`.
    """

    mode: Literal["off", "automatic"]
    """Specifies the recording behavior for the live input.

    Set this value to `off` to prevent a recording. Set the value to `automatic` to
    begin a recording and transition to on-demand after Stream Live stops receiving
    input.
    """

    require_signed_urls: Annotated[bool, PropertyInfo(alias="requireSignedURLs")]
    """
    Indicates if a video using the live input has the `requireSignedURLs` property
    set. Also enforces access controls on any video recording of the livestream with
    the live input.
    """

    timeout_seconds: Annotated[int, PropertyInfo(alias="timeoutSeconds")]
    """
    Determines the amount of time a live input configured in `automatic` mode should
    wait before a recording transitions from live to on-demand. `0` is recommended
    for most use cases and indicates the platform default should be used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Type, Optional, cast

import httpx

from .dns import (
    DNSResource,
    AsyncDNSResource,
    DNSResourceWithRawResponse,
    AsyncDNSResourceWithRawResponse,
    DNSResourceWithStreamingResponse,
    AsyncDNSResourceWithStreamingResponse,
)
from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .addresses import (
    AddressesResource,
    AsyncAddressesResource,
    AddressesResourceWithRawResponse,
    AsyncAddressesResourceWithRawResponse,
    AddressesResourceWithStreamingResponse,
    AsyncAddressesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from .rules.rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from ...types.email_routing import email_routing_enable_params, email_routing_disable_params
from ...types.email_routing.settings import Settings

__all__ = ["EmailRoutingResource", "AsyncEmailRoutingResource"]


class EmailRoutingResource(SyncAPIResource):
    @cached_property
    def dns(self) -> DNSResource:
        return DNSResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def addresses(self) -> AddressesResource:
        return AddressesResource(self._client)

    @cached_property
    def with_raw_response(self) -> EmailRoutingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EmailRoutingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EmailRoutingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EmailRoutingResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    def disable(
        self,
        *,
        zone_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """Disable your Email Routing zone.

        Also removes additional MX records previously
        required for Email Routing to work.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/email/routing/disable", zone_id=zone_id),
            body=maybe_transform(body, email_routing_disable_params.EmailRoutingDisableParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )

    @typing_extensions.deprecated("deprecated")
    def enable(
        self,
        *,
        zone_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """Enable you Email Routing zone.

        Add and lock the necessary MX and SPF records.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/email/routing/enable", zone_id=zone_id),
            body=maybe_transform(body, email_routing_enable_params.EmailRoutingEnableParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """
        Get information about the settings for your Email Routing zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/email/routing", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )


class AsyncEmailRoutingResource(AsyncAPIResource):
    @cached_property
    def dns(self) -> AsyncDNSResource:
        return AsyncDNSResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def addresses(self) -> AsyncAddressesResource:
        return AsyncAddressesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEmailRoutingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEmailRoutingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEmailRoutingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEmailRoutingResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("deprecated")
    async def disable(
        self,
        *,
        zone_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """Disable your Email Routing zone.

        Also removes additional MX records previously
        required for Email Routing to work.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/email/routing/disable", zone_id=zone_id),
            body=await async_maybe_transform(body, email_routing_disable_params.EmailRoutingDisableParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )

    @typing_extensions.deprecated("deprecated")
    async def enable(
        self,
        *,
        zone_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """Enable you Email Routing zone.

        Add and lock the necessary MX and SPF records.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/email/routing/enable", zone_id=zone_id),
            body=await async_maybe_transform(body, email_routing_enable_params.EmailRoutingEnableParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Settings]:
        """
        Get information about the settings for your Email Routing zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/email/routing", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Settings]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Settings]], ResultWrapper[Settings]),
        )


class EmailRoutingResourceWithRawResponse:
    def __init__(self, email_routing: EmailRoutingResource) -> None:
        self._email_routing = email_routing

        self.disable = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                email_routing.disable,  # pyright: ignore[reportDeprecated],
            )
        )
        self.enable = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                email_routing.enable,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = to_raw_response_wrapper(
            email_routing.get,
        )

    @cached_property
    def dns(self) -> DNSResourceWithRawResponse:
        return DNSResourceWithRawResponse(self._email_routing.dns)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._email_routing.rules)

    @cached_property
    def addresses(self) -> AddressesResourceWithRawResponse:
        return AddressesResourceWithRawResponse(self._email_routing.addresses)


class AsyncEmailRoutingResourceWithRawResponse:
    def __init__(self, email_routing: AsyncEmailRoutingResource) -> None:
        self._email_routing = email_routing

        self.disable = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                email_routing.disable,  # pyright: ignore[reportDeprecated],
            )
        )
        self.enable = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                email_routing.enable,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = async_to_raw_response_wrapper(
            email_routing.get,
        )

    @cached_property
    def dns(self) -> AsyncDNSResourceWithRawResponse:
        return AsyncDNSResourceWithRawResponse(self._email_routing.dns)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._email_routing.rules)

    @cached_property
    def addresses(self) -> AsyncAddressesResourceWithRawResponse:
        return AsyncAddressesResourceWithRawResponse(self._email_routing.addresses)


class EmailRoutingResourceWithStreamingResponse:
    def __init__(self, email_routing: EmailRoutingResource) -> None:
        self._email_routing = email_routing

        self.disable = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                email_routing.disable,  # pyright: ignore[reportDeprecated],
            )
        )
        self.enable = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                email_routing.enable,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = to_streamed_response_wrapper(
            email_routing.get,
        )

    @cached_property
    def dns(self) -> DNSResourceWithStreamingResponse:
        return DNSResourceWithStreamingResponse(self._email_routing.dns)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._email_routing.rules)

    @cached_property
    def addresses(self) -> AddressesResourceWithStreamingResponse:
        return AddressesResourceWithStreamingResponse(self._email_routing.addresses)


class AsyncEmailRoutingResourceWithStreamingResponse:
    def __init__(self, email_routing: AsyncEmailRoutingResource) -> None:
        self._email_routing = email_routing

        self.disable = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                email_routing.disable,  # pyright: ignore[reportDeprecated],
            )
        )
        self.enable = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                email_routing.enable,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get = async_to_streamed_response_wrapper(
            email_routing.get,
        )

    @cached_property
    def dns(self) -> AsyncDNSResourceWithStreamingResponse:
        return AsyncDNSResourceWithStreamingResponse(self._email_routing.dns)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._email_routing.rules)

    @cached_property
    def addresses(self) -> AsyncAddressesResourceWithStreamingResponse:
        return AsyncAddressesResourceWithStreamingResponse(self._email_routing.addresses)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["LiveStat"]


class LiveStat(BaseModel):
    unique_devices_total: Optional[float] = FieldInfo(alias="uniqueDevicesTotal", default=None)
    """Number of unique devices"""

    value: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ase_origin_params import AseOriginParams as AseOriginParams
from .ase_origin_response import AseOriginResponse as AseOriginResponse
from .location_origin_params import LocationOriginParams as LocationOriginParams
from .location_target_params import LocationTargetParams as LocationTargetParams
from .location_origin_response import LocationOriginResponse as LocationOriginResponse
from .location_target_response import LocationTargetResponse as LocationTargetResponse
